﻿using ComPDFKit.PDFDocument;
using ComPDFKitViewer.PdfViewer;
using PDF_Office.EventAggregators;
using PDF_Office.Helper;
using PDF_Office.Model;
using PDF_Office.Model.EditTools.Background;
using PDF_Office.Model.EditTools.Watermark;
using PDF_Office.Properties;
using PDFSettings;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Media;

namespace PDF_Office.ViewModels.EditTools.Background
{

    public class BackgroundCreateColorContentViewModel : BindableBase, INavigationAware
    {

        IEventAggregator eventAggregator;
        private CPDFViewer PDFViewer;

        private string RedColorBtn = "RedColorBtn";
        private string GreenColorBtn = "GreenColorBtn";



        private List<string> _rotationList = new List<string>();
        public List<string> RotationList
        {
            get { return _rotationList; }
            set { SetProperty(ref _rotationList, value); }
        }

        private List<string> _opacityList = new List<string>();
        public List<string> OpacityList
        {
            get { return _opacityList; }
            set
            {
                SetProperty(ref _opacityList, value);
            }
        }

        private List<string> _relativeRatioList = new List<string>();
        public List<string> RelativeRatioList
        {
            get { return _relativeRatioList; }
            set
            {
                SetProperty(ref _relativeRatioList, value);
            }
        }

        private int _rotationValue = 0;
        public int RotationValue
        {
            get { return _rotationValue; }
            set
            {
                SetProperty(ref _rotationValue, value);
                BackgroundInfo.Rotation = value;
                eventAggregator.GetEvent<SetBackgroundEvent>().Publish(BackgroundInfo);
            }
        }

        private int _opacityValue = 100;
        public int OpacityValue
        {
            get { return _opacityValue; }
            set
            {
                SetProperty(ref _opacityValue, value);
                BackgroundInfo.Opacity = (byte)(((float)value / 100) * 255);
                eventAggregator.GetEvent<SetBackgroundEvent>().Publish(BackgroundInfo);
            }
        }

        private int _relativeRatioValue = 0;
        public int RelativeRatioValue
        {
            get { return _relativeRatioValue; }
            set
            {
                SetProperty(ref _rotationValue, value);
                BackgroundInfo.RelativeRatio = value;
            }
        }

        private string _verticalOffsetValue = "0";
        public string VerticalOffsetValue
        {
            get { return _verticalOffsetValue; }
            set
            {
                SetProperty(ref _verticalOffsetValue, value);
                BackgroundInfo.VertOffset = float.Parse(VerticalOffsetValue);
                eventAggregator.GetEvent<SetBackgroundEvent>().Publish(BackgroundInfo);
            }
        }

        private string _horizontalOffset = "0";
        public string HorizontalOffset
        {
            get { return _horizontalOffset; }
            set
            {
                SetProperty(ref _horizontalOffset, value);
                BackgroundInfo.HorizOffset = float.Parse(HorizontalOffset);
                eventAggregator.GetEvent<SetBackgroundEvent>().Publish(BackgroundInfo);
            }
        }



        private ObservableDictionary<string, bool> _getLocationFromNumber = new ObservableDictionary<string, bool>();
        public ObservableDictionary<string, bool> GetLocationFromNumber
        {
            get { return _getLocationFromNumber; }
        }

        private BackgroundInfo BackgroundInfo = new BackgroundInfo();

        private Color _color = Color.FromArgb(0xFF, 0xFF, 0x00, 0x00);
        public Color Color
        {
            get
            {
                return _color;
            }
            set
            {
                SetProperty(ref _color, value);
                BackgroundInfo.Color = EditToolsHelper.ConvertColor(value);
                eventAggregator.GetEvent<SetBackgroundEvent>().Publish(BackgroundInfo);
            }
        }

        private string _pageRangeText { get; set; } = "0";
        public string PageRangeText
        {
            get { return _pageRangeText; }
            set
            {
                _pageRangeText = value;
                EditToolsHelper.GetPageRange(PageRangeSelectIndex, PDFViewer.Document, ref BackgroundInfo.PageRange, PageRangeText);
                eventAggregator.GetEvent<SetBackgroundEvent>().Publish(BackgroundInfo);
            }
        }

        private int _pageRangeSelectIndex = 0;
        public int PageRangeSelectIndex
        {
            get
            {
                return _pageRangeSelectIndex;
            }
            set
            {
                SetProperty(ref _pageRangeSelectIndex, value);
                EditToolsHelper.GetPageRange(PageRangeSelectIndex, PDFViewer.Document, ref BackgroundInfo.PageRange, PageRangeText);
                eventAggregator.GetEvent<SetBackgroundEvent>().Publish(BackgroundInfo); 
            }
        }



        public DelegateCommand<object> ChangeLocationCommand { get; set; }
        public DelegateCommand<object> ChangeColorCommand { get; set; }
        public DelegateCommand<object> SelectedColorCommand { get; set; }

        public BackgroundCreateColorContentViewModel(IEventAggregator eventAggregator)
        {
            this.eventAggregator = eventAggregator;
            ChangeLocationCommand = new DelegateCommand<object>(ChangeLocation);
            ChangeColorCommand = new DelegateCommand<object>(ChangeColor);
            BackgroundInfo.Color = EditToolsHelper.ConvertColor(Color.FromArgb(0xFF, 0xFF, 0x00, 0x00));
            InitComponent();
            BackgroundInfo.Horizalign = C_Background_Horizalign.BG_HORIZALIGN_CENTER;
            BackgroundInfo.Vertalign = C_Background_Vertalign.BG_VERTALIGN_CENTER;
            InitLocationButtonMatrix();
            eventAggregator.GetEvent<SaveBackgroundTemplateEvent>().Subscribe(SaveBackgroundTemplate);
        }

        public void InitComponent()
        {
            InitOpacityList();
            InitRotationList();
            InitRelativeRatioList();
        }

        private void InitRotationList()
        {
            OpacityList.Clear();
            for (int defaultRotation = -45; defaultRotation <= 45; defaultRotation += 15)
            {
                RotationList.Add(defaultRotation.ToString());
            }
        }

        private void InitOpacityList()
        {
            OpacityList.Clear();
            for (int defaultOpacity = 10; defaultOpacity <= 100; defaultOpacity += 10)
            {
                OpacityList.Add(defaultOpacity.ToString() + " %");
            }
        }

        private void InitRelativeRatioList()
        {
            RelativeRatioList.Clear();
            for (int defaultRelativeRatioList = 10; defaultRelativeRatioList <= 100; defaultRelativeRatioList += 10)
            {
                RelativeRatioList.Add(defaultRelativeRatioList.ToString() + " %");
            }
        }

        private void InitLocationButtonMatrix()
        {
            GetLocationFromNumber.Clear();
            for (var temp = 0; temp <= 22; temp++)
            {
                GetLocationFromNumber.Add(temp.ToString(), true);
                if (temp % 10 == 2)
                {
                    temp += 7;
                }
            }
            int Num = (int)BackgroundInfo.Vertalign * 10 + (int)BackgroundInfo.Horizalign;
            GetLocationFromNumber[Num.ToString()] = false;
        }

        public void ChangeLocation(object e)
        {
            string args = e as string;
            if (args != null)
            {
                BackgroundInfo.Vertalign = (C_Background_Vertalign)(int.Parse(args) / 10);
                BackgroundInfo.Horizalign = (C_Background_Horizalign)(int.Parse(args) % 10);
                eventAggregator.GetEvent<SetBackgroundEvent>().Publish(BackgroundInfo);
                InitLocationButtonMatrix();
            }
        }

        public void ChangeColor(object e)
        {
            if (e != null)
            {
                var colorValue = (Color)e;
                if (colorValue != null)
                {
                    Color = colorValue;
                }
            }
        }

        public void SaveCurrentTemplate()
        {
                var backgroundItem = new BackgroundItem();
                backgroundItem.pageRange = "0";
                backgroundItem.type = ComPDFKit.PDFDocument.C_Background_Type.BG_TYPE_COLOR;
                backgroundItem.templateName += Settings.Default.BackgroundIndex.ToString();
                Settings.Default.BackgroundTemplateList.Add(backgroundItem);
                Settings.Default.Save();
                this.eventAggregator.GetEvent<EnterTemplateListOrCreateEvent>().Publish(EnumTemplateListOrCreate.StatusTemplate);
        }

        public void SaveBackgroundTemplate(EnumColorOrFile enumColorOrFile)
        {
            if(enumColorOrFile == EnumColorOrFile.StatusColor)
            {
                SaveCurrentTemplate();
            }
        }
        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }

        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            navigationContext.Parameters.TryGetValue<CPDFViewer>(ParameterNames.PDFViewer, out PDFViewer);
            InitOpacityList();
        }
    }
}
